==> mp3plot-0.5.1.tar.gz: mp3plot-0.5.1/src/bitrate_dist.cc <==
/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: bitrate_dist.cc 1637 2009-04-14 02:10:48Z  $

#include <cassert>

#include "mp3/bitrate.h"
#include "mp3/frame.h"
#include "bitrate_dist.h"

namespace mp3plot {

using mp3::standard_bitrates;

bitrate_distribution::bitrate_distribution()
    : max(0),
      is_filled(false)
{
    const standard_bitrates::bitrate_vector & v = standard_bitrates::as_vector();
    for (standard_bitrates::bitrate_vector::const_iterator it = v.begin();
         it != v.end();
         ++it)
    {
        table[ *it ] = 0;
    }
}

void bitrate_distribution::fill_from(const frame_collection & frames, bool is_vbr) {
    assert( ! is_filled );
    assert( 0 == max );

    frame_collection::const_iterator it = frames.begin();

    // The first frame *on VBR files* doesn't contain audio.
    // It just contains meta info.
    if (is_vbr) {
        ++it;
    }
    for ( ; it != frames.end(); ++it) {
        table[it->bitrate()]++;
        if (max < table[it->bitrate()]) { max=table[it->bitrate()]; }
    }

    is_filled = true;
}

bitrate_distribution::const_iterator bitrate_distribution::begin() const {
     assert( is_filled );

     // bitrate_table_t is sorted, the invalid value is < 0, so it will be
     // at the start, jump over it
     bitrate_map_base_iterator_t it = table.begin();

     // Let's self-protect from future changes...
     assert( it->first == standard_bitrates::bad_bitrate() );
     it++;
     assert( it->first >= 0 );

     return bitrate_table_iterator(it);
}

numframes_t bitrate_distribution::bad_frames() const {
    assert( is_filled );

    bitrate_map_base_iterator_t it = table.begin();
    // As in begin(), bad_bitrate() should be at the start and have a negative key
    assert( it->first == standard_bitrates::bad_bitrate() );

    return it->second;
}

} // namespace mp3plot

// vim:set ts=4 et ai:

==> mp3plot-0.6.0.tar.gz: mp3plot-0.6.0/src/bitrate_dist.cc <==
/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: bitrate_dist.cc 1342 2009-06-21 23:41:45Z toni $

#include <cassert>

#include "mp3/bitrate.h"
#include "mp3/frame.h"
#include "bitrate_dist.h"

#include <iostream>

namespace mp3plot {

using mp3::bitrate_translator;

bitrate_distribution::bitrate_distribution()
    : max(0),
      is_filled(false)
{
    for (uint8_t i = 0; i<bitrate_translator::number(); ++i) {
        table[i] = 0;
    }
}

void bitrate_distribution::fill_from(const frame_collection & frames, bool is_vbr) {
    assert( ! is_filled );
    assert( 0 == max );

    frame_collection::const_iterator it = frames.begin();

    // The first frame *on VBR files* doesn't contain audio.
    // It just contains meta info.
    const mp3::frame & f = *frames.begin();
    const mp3::frame_first * pff = reinterpret_cast<const mp3::frame_first*>(&f);

    // If the first frame is tainted it might or might not contain
    // actual audio
    if (is_vbr && !pff->is_tainted()) {
        ++it;
    }
    for ( ; it != frames.end(); ++it) {
        uint8_t idx = it->get_bitrate_index();
        assert( idx < 0x10 ); // 4 bits max
        table[idx]++;
        if (max < table[idx]) { max=table[idx]; }
    }

    is_filled = true;
}

bitrate_distribution::const_iterator bitrate_distribution::begin() const {
     assert( is_filled );

     // bitrate_table_t is sorted, the invalid value is < 0, so it will be
     // at the start, jump over it
     bitrate_map_base_iterator_t it = table.begin();

     // Let's self-protect from future changes...
#if 0
     assert( it->first == standard_bitrates::bad_bitrate() );
     it++;
     assert( it->first >= 0 );
#endif

     return bitrate_table_iterator(it);
}

numframes_t bitrate_distribution::bad_frames() const {
    assert( is_filled );

    bitrate_map_base_iterator_t it = table.begin();
#if 0
    // As in begin(), bad_bitrate() should be at the start and have a negative key
    assert( it->first == standard_bitrates::bad_bitrate() );
#endif

    return it->second;
}

} // namespace mp3plot

// vim:set ts=4 et ai:
